@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2 of the License, or (at your
+ * Free Software Foundation; either version 3 of the License, or (at your
  * option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
@@ -21,16 +21,27 @@
 
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
 #include <string.h>
 
+#if GTK_CHECK_VERSION (2, 90, 7)
+#define GDK_KEY(symbol) GDK_KEY_##symbol
+#else
+#include <gdk/gdkkeysyms.h>
+#define GDK_KEY(symbol) GDK_##symbol
+#endif
+
 #include "gucharmap-charmap.h"
 #include "gucharmap-unicode-info.h"
 #include "gucharmap-marshal.h"
 #include "gucharmap-private.h"
 
 struct _GucharmapCharmapPrivate {
+#if !GTK_CHECK_VERSION (2, 15, 0)
+  GtkOrientation orientation;
+  GtkWidgetClass *paned_class;
+#endif
+
   GtkWidget *notebook;
   GucharmapChaptersView *chapters_view;
   GucharmapChartable *chartable;
@@ -58,13 +69,17 @@ enum
 
 enum {
   PROP_0,
+#if !GTK_CHECK_VERSION (2, 15, 0)
+  PROP_ORIENTATION,
+#endif
   PROP_CHAPTERS_MODEL,
   PROP_ACTIVE_CHAPTER,
   PROP_ACTIVE_CHARACTER,
   PROP_ACTIVE_CODEPOINT_LIST,
   PROP_ACTIVE_PAGE,
   PROP_SNAP_POW2,
-  PROP_FONT_DESC
+  PROP_FONT_DESC,
+  PROP_FONT_FALLBACK
 };
 
 static guint gucharmap_charmap_signals[NUM_SIGNALS];
@@ -72,7 +87,7 @@ static guint gucharmap_charmap_signals[NUM_SIGNALS];
 static void gucharmap_charmap_class_init (GucharmapCharmapClass *klass);
 static void gucharmap_charmap_init       (GucharmapCharmap *charmap);
 
-G_DEFINE_TYPE (GucharmapCharmap, gucharmap_charmap, GTK_TYPE_HPANED)
+G_DEFINE_TYPE (GucharmapCharmap, gucharmap_charmap, GTK_TYPE_PANED)
 
 static void
 gucharmap_charmap_finalize (GObject *object)
@@ -86,6 +101,10 @@ gucharmap_charmap_finalize (GObject *object)
   if (priv->font_desc)
     pango_font_description_free (priv->font_desc);
 
+#if !GTK_CHECK_VERSION (2, 15, 0)
+  g_type_class_unref (priv->paned_class);
+#endif
+
   G_OBJECT_CLASS (gucharmap_charmap_parent_class)->finalize (object);
 }
 
@@ -99,6 +118,11 @@ gucharmap_charmap_get_property (GObject *object,
   GucharmapCharmapPrivate *priv = charmap->priv;
 
   switch (prop_id) {
+#if !GTK_CHECK_VERSION (2, 15, 0)
+    case PROP_ORIENTATION:
+      g_value_set_enum (value, gucharmap_charmap_get_orientation (charmap));
+      break;
+#endif
     case PROP_CHAPTERS_MODEL:
       g_value_set_object (value, gucharmap_charmap_get_chapters_model (charmap));
       break;
@@ -117,6 +141,9 @@ gucharmap_charmap_get_property (GObject *object,
     case PROP_FONT_DESC:
       g_value_set_boxed (value, gucharmap_charmap_get_font_desc (charmap));
       break;
+    case PROP_FONT_FALLBACK:
+      g_value_set_boolean (value, gucharmap_charmap_get_font_fallback (charmap));
+      break;
     case PROP_SNAP_POW2:
       g_value_set_boolean (value, gucharmap_charmap_get_snap_pow2 (charmap));
       break;
@@ -136,6 +163,11 @@ gucharmap_charmap_set_property (GObject *object,
   GucharmapCharmapPrivate *priv = charmap->priv;
 
   switch (prop_id) {
+#if !GTK_CHECK_VERSION (2, 15, 0)
+    case PROP_ORIENTATION:
+      gucharmap_charmap_set_orientation (charmap, g_value_get_enum (value));
+      break;
+#endif
     case PROP_CHAPTERS_MODEL:
       gucharmap_charmap_set_chapters_model (charmap, g_value_get_object (value));
       break;
@@ -152,6 +184,9 @@ gucharmap_charmap_set_property (GObject *object,
     case PROP_FONT_DESC:
       gucharmap_charmap_set_font_desc (charmap, g_value_get_boxed (value));
       break;
+    case PROP_FONT_FALLBACK:
+      gucharmap_charmap_set_font_fallback (charmap, g_value_get_boolean (value));
+      break;
     case PROP_SNAP_POW2:
       gucharmap_charmap_set_snap_pow2 (charmap, g_value_get_boolean (value));
       break;
@@ -162,10 +197,37 @@ gucharmap_charmap_set_property (GObject *object,
   }
 }
 
+#if !GTK_CHECK_VERSION (2, 15, 0)
+
 static void
-gucharmap_charmap_class_init (GucharmapCharmapClass *clazz)
+gucharmap_charmap_size_request (GtkWidget *widget,
+                                GtkRequisition *requisition)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (clazz);
+  GucharmapCharmap *charmap = GUCHARMAP_CHARMAP (widget);
+  GucharmapCharmapPrivate *priv = charmap->priv;
+
+  priv->paned_class->size_request (widget, requisition);
+}
+
+static void
+gucharmap_charmap_size_allocate (GtkWidget *widget,
+                                 GtkAllocation  *allocation)
+{
+  GucharmapCharmap *charmap = GUCHARMAP_CHARMAP (widget);
+  GucharmapCharmapPrivate *priv = charmap->priv;
+
+  priv->paned_class->size_allocate (widget, allocation);
+}
+
+#endif /* GTK < 2.15.0 */
+
+static void
+gucharmap_charmap_class_init (GucharmapCharmapClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+#if !GTK_CHECK_VERSION (2, 15, 0)
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+#endif
 
   _gucharmap_intl_ensure_initialized ();
 
@@ -173,6 +235,11 @@ gucharmap_charmap_class_init (GucharmapCharmapClass *clazz)
   object_class->set_property = gucharmap_charmap_set_property;
   object_class->finalize = gucharmap_charmap_finalize;
 
+#if !GTK_CHECK_VERSION (2, 15, 0)
+  widget_class->size_request = gucharmap_charmap_size_request;
+  widget_class->size_allocate = gucharmap_charmap_size_allocate;
+#endif
+
   gucharmap_charmap_signals[STATUS_MESSAGE] =
       g_signal_new (I_("status-message"), gucharmap_charmap_get_type (),
                     G_SIGNAL_RUN_FIRST,
@@ -187,6 +254,20 @@ gucharmap_charmap_class_init (GucharmapCharmapClass *clazz)
                     NULL, NULL, _gucharmap_marshal_VOID__UINT_UINT, G_TYPE_NONE, 
                     2, G_TYPE_UINT, G_TYPE_UINT);
 
+#if !GTK_CHECK_VERSION (2, 15, 0)
+  g_object_class_install_property
+    (object_class,
+     PROP_ORIENTATION,
+     g_param_spec_enum ("orientation", NULL, NULL,
+                        GTK_TYPE_ORIENTATION,
+                        GTK_ORIENTATION_HORIZONTAL,
+                        G_PARAM_READWRITE |
+                        G_PARAM_CONSTRUCT |
+                        G_PARAM_STATIC_NAME |
+                        G_PARAM_STATIC_NICK |
+                        G_PARAM_STATIC_BLURB));
+#endif
+
   g_object_class_install_property
     (object_class,
      PROP_CHAPTERS_MODEL,
@@ -252,6 +333,20 @@ gucharmap_charmap_class_init (GucharmapCharmapClass *clazz)
                          G_PARAM_STATIC_NICK |
                          G_PARAM_STATIC_BLURB));
 
+  /**
+   * GucharmapCharmap:font-fallback:
+   *
+   * Whether font fallback is enabled.
+   *
+   * Since: 2.34
+   */
+  g_object_class_install_property
+    (object_class,
+     PROP_FONT_FALLBACK,
+     g_param_spec_boolean ("font-fallback", NULL, NULL,
+                           TRUE,
+                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_property
     (object_class,
      PROP_SNAP_POW2,
@@ -869,9 +964,9 @@ details_key_press_event (GtkWidget *text_view,
 
   switch (event->keyval)
     {
-      case GDK_Return:
-      case GDK_ISO_Enter:
-      case GDK_KP_Enter:
+      case GDK_KEY (Return):
+      case GDK_KEY (ISO_Enter):
+      case GDK_KEY (KP_Enter):
         buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
         gtk_text_buffer_get_iter_at_mark (buffer, &iter, 
                                           gtk_text_buffer_get_insert (buffer));
@@ -981,7 +1076,7 @@ details_motion_notify_event (GtkWidget *text_view,
 
   set_cursor_if_appropriate (charmap, x, y);
 
-  gdk_window_get_pointer (text_view->window, NULL, NULL, NULL);
+  gdk_window_get_pointer (gtk_widget_get_window (text_view), NULL, NULL, NULL);
   return FALSE;
 }
 
@@ -992,7 +1087,7 @@ details_visibility_notify_event (GtkWidget *text_view,
 {
   gint wx, wy, bx, by;
 
-  gdk_window_get_pointer (text_view->window, &wx, &wy, NULL);
+  gdk_window_get_pointer (gtk_widget_get_window (text_view), &wx, &wy, NULL);
 
   gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), 
                                          GTK_TEXT_WINDOW_WIDGET,
@@ -1005,7 +1100,11 @@ details_visibility_notify_event (GtkWidget *text_view,
 
 static void
 notebook_switch_page (GtkNotebook *notebook,
-                      GtkNotebookPage *page /* useless */,
+#if GTK_CHECK_VERSION (2, 90, 6)
+                      GtkWidget *page,
+#else
+                      gpointer page /* useless */,
+#endif
                       guint page_num,
                       GucharmapCharmap *charmap)
 {
@@ -1121,6 +1220,7 @@ gucharmap_charmap_init (GucharmapCharmap *charmap)
   textview = gtk_text_view_new ();
   priv->details_view = GTK_TEXT_VIEW (textview);
   gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
+  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview), FALSE);
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview),
                                GTK_WRAP_WORD);
 
@@ -1169,12 +1269,80 @@ gucharmap_charmap_init (GucharmapCharmap *charmap)
 GtkWidget *
 gucharmap_charmap_new (void)
 {
-  return g_object_new (gucharmap_charmap_get_type (), NULL);
+  return g_object_new (GUCHARMAP_TYPE_CHARMAP,
+                       "orientation", GTK_ORIENTATION_HORIZONTAL,
+                       NULL);
+}
+
+#ifndef GUCHARMAP_DISABLE_DEPRECATED_SOURCE
+
+/**
+ * gucharmap_charmap_set_orientation:
+ * @charmap:
+ * @orientation:
+ *
+ * Deprecated: 2.25.0
+ */
+void
+gucharmap_charmap_set_orientation (GucharmapCharmap *charmap,
+                                   GtkOrientation orientation)
+{
+#if GTK_CHECK_VERSION (2, 15, 0)
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (charmap), orientation);
+#else
+  GucharmapCharmapPrivate *priv;
+  GtkPaned *paned = GTK_PANED (charmap);
+
+  g_return_if_fail (GUCHARMAP_IS_CHARMAP (charmap));
+  priv = charmap->priv;
+
+  priv->orientation = orientation;
+
+  if (priv->paned_class)
+    g_type_class_unref (priv->paned_class);
+
+  switch (orientation) {
+    case GTK_ORIENTATION_HORIZONTAL:
+      priv->paned_class = g_type_class_ref (GTK_TYPE_HPANED);
+
+      paned->cursor_type = GDK_SB_H_DOUBLE_ARROW;
+      paned->orientation = GTK_ORIENTATION_VERTICAL;
+      break;
+    case GTK_ORIENTATION_VERTICAL:
+      priv->paned_class = g_type_class_ref (GTK_TYPE_VPANED);
+
+      paned->cursor_type = GDK_SB_V_DOUBLE_ARROW;
+      paned->orientation = GTK_ORIENTATION_HORIZONTAL;
+      break;
+  }
+
+  g_object_notify (G_OBJECT (charmap), "orientation");
+#endif
 }
 
 /**
- * gucharmap_chartable_set_font_desc:
- * @chartable: a #GucharmapChartable
+ * gucharmap_charmap_get_orientation:
+ * @charmap:
+ *
+ * Deprecated: 2.25.0
+ */
+GtkOrientation
+gucharmap_charmap_get_orientation (GucharmapCharmap *charmap)
+{
+  g_return_val_if_fail (GUCHARMAP_IS_CHARMAP (charmap), GTK_ORIENTATION_HORIZONTAL);
+
+#if GTK_CHECK_VERSION (2, 15, 0)
+  return gtk_orientable_get_orientation (GTK_ORIENTABLE (charmap));
+#else
+  return charmap->priv->orientation;
+#endif
+}
+
+#endif /* !GUCHARMAP_DISABLE_DEPRECATED_SOURCE */
+
+/**
+ * gucharmap_charmap_set_font_desc:
+ * @charmap:
  * @font_desc: a #PangoFontDescription
  *
  * Sets @font_desc as the font to use to display the character table.
@@ -1213,6 +1381,41 @@ gucharmap_charmap_get_font_desc (GucharmapCharmap *charmap)
   return charmap->priv->font_desc;
 }
 
+/**
+ * gucharmap_charmap_set_font_fallback:
+ * @charmap: a #GucharmapCharmap
+ * @enable_font_fallback: whether to enable font fallback
+ *
+ * Since: 2.34
+ */
+void 
+gucharmap_charmap_set_font_fallback (GucharmapCharmap *charmap,
+                                     gboolean enable_font_fallback)
+{
+  g_return_if_fail (GUCHARMAP_IS_CHARMAP (charmap));
+
+  gucharmap_chartable_set_font_fallback (charmap->priv->chartable, 
+                                         enable_font_fallback);
+
+  g_object_notify (G_OBJECT (charmap), "font-fallback");
+}
+
+/**
+ * gucharmap_charmap_get_font_fallback:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Returns: whether font fallback is enabled
+ *
+ * Since: 2.34
+ */
+gboolean
+gucharmap_charmap_get_font_fallback (GucharmapCharmap *charmap)
+{
+  g_return_val_if_fail (GUCHARMAP_IS_CHARMAP (charmap), FALSE);
+
+  return gucharmap_chartable_get_font_fallback (charmap->priv->chartable);
+}
+
 void
 gucharmap_charmap_set_active_character (GucharmapCharmap *charmap,
                                         gunichar          wc)
@@ -1272,6 +1475,12 @@ gucharmap_charmap_previous_chapter (GucharmapCharmap *charmap)
   gucharmap_chapters_view_previous (priv->chapters_view);
 }
 
+/**
+ * gucharmap_charmap_get_chartable:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Returns: (transfer none): a #GucharmapChartable
+ */
 GucharmapChartable *
 gucharmap_charmap_get_chartable (GucharmapCharmap *charmap)
 {
@@ -1308,6 +1517,15 @@ gucharmap_charmap_set_chapters_model (GucharmapCharmap  *charmap,
   g_object_thaw_notify (object);
 }
 
+/**
+ * gucharmap_charmap_get_chapters_model:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Gets the #GucharmapChaptersModel associated with the #GucharmapChaptersView
+ * of @charmap.
+ *
+ * Returns: (transfer none): a #GucharmapChaptersModel
+ */
 GucharmapChaptersModel *
 gucharmap_charmap_get_chapters_model (GucharmapCharmap *charmap)
 {
@@ -1316,14 +1534,29 @@ gucharmap_charmap_get_chapters_model (GucharmapCharmap *charmap)
   return gucharmap_chapters_view_get_model (priv->chapters_view);
 }
 
+/**
+ * gucharmap_charmap_get_chapters_view:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Gets the #GucharmapChaptersView of @charmap
+ *
+ * Returns: (transfer none): the #GucharmapChaptersView
+ */
 GucharmapChaptersView *
-gucharmap_charmap_get_chapters_view  (GucharmapCharmap *charmap)
+gucharmap_charmap_get_chapters_view (GucharmapCharmap *charmap)
 {
   GucharmapCharmapPrivate *priv = charmap->priv;
 
   return priv->chapters_view;
 }
 
+/**
+ * gucharmap_charmap_get_book_codepoint_list:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Returns: (transfer full): the GucharmapCodepointList. Must be freed with
+ * g_object_unref().
+ */
 GucharmapCodepointList *
 gucharmap_charmap_get_book_codepoint_list (GucharmapCharmap *charmap)
 {
@@ -1346,7 +1579,11 @@ gucharmap_charmap_get_chapters_visible (GucharmapCharmap *charmap)
 {
   GucharmapCharmapPrivate *priv = charmap->priv;
 
+#if GTK_CHECK_VERSION (2,18,0)
+  return gtk_widget_get_visible (GTK_WIDGET (priv->chapters_view));
+#else
   return GTK_WIDGET_VISIBLE (priv->chapters_view);
+#endif
 }
 
 void
@@ -1375,7 +1612,11 @@ gucharmap_charmap_get_page_visible (GucharmapCharmap *charmap,
   if (!page_widget)
     return FALSE;
 
+#if GTK_CHECK_VERSION (2,18,0)
+  return gtk_widget_get_visible (page_widget);
+#else
   return GTK_WIDGET_VISIBLE (page_widget);
+#endif
 }
 
 void
@@ -1395,6 +1636,15 @@ gucharmap_charmap_get_active_page (GucharmapCharmap *charmap)
   return priv->active_page;
 }
 
+/**
+ * gucharmap_charmap_get_active_codepoint_list:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Gets the @GucharmapCodepointList associated with the #GucharmapChartable of
+ * @charmap.
+ *
+ * Returns: (transfer none): the #GucharmapCodepointList
+ */
 GucharmapCodepointList *
 gucharmap_charmap_get_active_codepoint_list (GucharmapCharmap *charmap)
 {
